#![doc = include_str!("../README.md")]
#[cfg(feature = "parallel")]
use rayon::prelude::{IntoParallelRefIterator, ParallelIterator};
use std::{collections::HashMap, fs, io::BufWriter, sync::Mutex};

mod checksum;
mod error;
mod file;
mod filetype;
pub(crate) mod nix;
mod node;
mod special;
mod stream;

pub use {
    checksum::{Algorithm, Checksum},
    error::Error,
    file::File,
    filetype::FileType,
    node::Node,
    special::Special,
    stream::Stream,
};

pub fn create_archive(path: &str, files: Vec<&str>, algorithm: Algorithm) -> Result<(), Error> {
    let fd = fs::OpenOptions::new()
        .create(true)
        .truncate(true)
        .open(path)?;
    let mut writer = BufWriter::new(fd);
    let links = Mutex::new(HashMap::new());
    for f in files.iter() {
        let node = Node::from_path(f, algorithm, &links)?;
        node.write(&mut writer)?;
    }
    Ok(())
}

#[cfg(feature = "parallel")]
pub fn par_create_archive(
    path: &str,
    files: Vec<String>,
    algorithm: Algorithm,
) -> Result<(), Error> {
    let links = Mutex::new(HashMap::<u64, String>::new());
    {
        let _fd = fs::File::create(path)?;
    }
    files.par_iter().try_for_each(|f| {
        let node = Node::from_path(f, algorithm, &links)?;
        let fd = fs::OpenOptions::new()
            .create(false)
            .truncate(false)
            .append(true)
            .open(path)?;
        let mut writer = BufWriter::new(fd);
        node.write(&mut writer)?;
        Ok::<(), Error>(())
    })?;
    Ok(())
}
